/**
 * Created on March 8, 2007
 * 
 * Tool for exporting CEM Business Process, Business Transaction statistic, Defects data and Incidents data through CEM webservices.
 * 
 * @copyright 1999-2007 CA.  All rights reserved.
 */

package com.export;

import java.util.Calendar;
import java.util.Date;

import org.apache.axis.transport.http.HTTPConstants;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.PostMethod;

import com.ca.wily.cemwebservices.ConfigurationDataOutServiceSoapBindingStub;
import com.ca.wily.cemwebservices.DTimeDuration;
import com.ca.wily.cemwebservices.EventsDataOutServiceSoapBindingStub;
import com.ca.wily.cemwebservices.IConfigurationDataOutServiceServiceLocator;
import com.ca.wily.cemwebservices.IEventsDataOutServiceServiceLocator;
import com.ca.wily.cemwebservices.IStatisticsDataOutServiceServiceLocator;
import com.ca.wily.cemwebservices.StatisticsDataOutServiceSoapBindingStub;
import static com.export.Constants.*;

/** 
 * CEMExportTool class contains samples for using CEM web service API to retrive 
 * Business Process statistic data, Business Transaction statistic data, 
 * Defects data and Incidents data.
 *
 * @author xinyi01
 */
public class CEMExportTool
{
    public static void main(String[] args) {
        
        String strCEMHost = null;
        String username = null;
        String password = null;
        String strPort = null;
        int AggregationType = 0;
        String strStartTime = null;
        String strEndTime = null;    
        int sampleNum = 0;
        String sampleCase = null;
        String user = null;
        String userGroup = null;
        String outPutFileName = null;
        String application = null;
        String businessProcess = null;
        String businessTransaction = null;
        String defectType = null;
        float[] throughputPercentiles = null;
        float[] timePercentiles = null;
        float[] sizePercentiles = null;
                
        try {
            
            if (args == null || args.length == 0 ){
                    Util.usage();
                    System.exit(-1);
            }
            
            for (int i = 0; i < args.length; i++) {
                if (args[i].compareToIgnoreCase("-host") == 0) {
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    strCEMHost = args[i];
                    System.out.println("CEM Host: " + strCEMHost);
                }else if (args[i].compareToIgnoreCase("-username") == 0) {
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    username = args[i];
                    System.out.println("CEM webservice login username: " + username);    
                }else if (args[i].compareToIgnoreCase("-password") == 0) {
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    password = args[i];
                    
                    /* The password read from command-line prompt is not secure.
                       Since this code is written in using JDK 1.5, so we don't 
                       secure the password.
                       In JDK1.6, there is method readPassword() from Console class 
                       which reads password from the console with echoing disabled.
                     */ 
                    System.out.println("CEM webservice longin password: " + "*******");
                }else if (args[i].compareToIgnoreCase("-port") == 0) {
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    strPort = args[i];
                    System.out.println("CEM webservice listen port: " + strPort);
                }else if (args[i].compareToIgnoreCase("-interval") == 0) {
                    i++;
                    if (i >= args.length) {
                        Util.usage();
                        System.exit(-1);
                    }
                    AggregationType = Integer.parseInt(args[i]);
                    switch(AggregationType){
                        case 1:
                            AggregationType = DTimeDuration.HOURLY;
                            break;
                        case 2:
                            AggregationType = DTimeDuration.DAILY;
                            break;
                        case 3:
                            AggregationType = DTimeDuration.WEEKLY;
                            break;
                        case 4:
                            AggregationType = DTimeDuration.MONTHLY;
                            break;
                        default:
                            AggregationType = DTimeDuration.HOURLY;
                            break;
                    }
                    System.out.println("Aggregation Type: " + AggregationType);
                }else if (args[i].compareToIgnoreCase("-start") == 0) {
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    strStartTime = new String(args[i]);
                    System.out.println("Query start time : " + strStartTime);
                }else if (args[i].compareToIgnoreCase("-end") == 0){
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    
                    strEndTime = new String(args[i]);
                    System.out.println("Query end time : " + strEndTime);
                }else if (args[i].compareToIgnoreCase("-command") == 0){    
                    i++;
                    if (i >= args.length) {
                        Util.usage();
                        System.exit(-1);
                    }
                    sampleNum = Integer.parseInt(args[i]);
                    switch(sampleNum){
                    case 1:
                        sampleCase = ALL_BP_STATS_DATA;
                        break;
                    case 2:
                        sampleCase = ALL_BT_STATS_DATA;
                        break;
                    case 3:
                        sampleCase = BP_STATS_DATA_BY_BP;
                        break;
                    case 4:
                        sampleCase = BT_STATS_DATA_BY_BT;
                        break;
                    case 5:
                        sampleCase = BP_STATS_DATA_BY_USER_GROUP;
                        break;
                    case 6:
                        sampleCase = BT_STATS_DATA_BY_USER_GROUP;
                        break;
                    case 7:
                        sampleCase = BP_STATS_DATA_BY_BP_USER_GROUP;
                        break;
                    case 8:
                        sampleCase = BT_STATS_DATA_BY_BT_USER_GROUP;
                        break;
                    case 9:
                        sampleCase = BT_STATS_DATA_BY_BP_BT_USER_GROUP;
                        break;
                    case 10:
                        sampleCase = DEFECTS_DATA_BY_BP;
                        break;
                    case 11:
                        sampleCase = DEFECTS_DATA_BY_BT;
                        break;
                    case 12:
                        sampleCase = DEFECTS_DATA_BY_USER_GROUP;
                        break;
                    case 13:
                        sampleCase = DEFECTS_DATA_BY_TYPE;
                        break;
                    case 14:
                        sampleCase = DEFECTS_DATA_BY_BP_BT_USER_GROUP_TYPE;                
                        break;
                    case 15:
                        sampleCase = INCIDENTS_DATA_BY_TIME_RANGE;
                        break;
                    case 16:
                        sampleCase = BP_STATS_DATA_BY_APP;
                        break;
                    case 17:
                        sampleCase = BT_STATS_DATA_BY_APP;
                        break;
                    case 18:
                        sampleCase = BP_STATS_DATA_BY_APP_USER_GROUP;
                        break;
                    case 19:
                        sampleCase = BT_STATS_DATA_BY_APP_USER_GROUP;
                        break;
                    case 21:
                        sampleCase = AGG_BP_STATS_DATA;
                        break;
                    case 22:
                        sampleCase = AGG_BT_STATS_DATA;
                        break;
                    case 23:
                        sampleCase = AGG_BP_STATS_DATA_BY_BP;
                        break;
                    case 24:
                        sampleCase = AGG_BT_STATS_DATA_BY_BT;
                        break;
                    case 25:
                        sampleCase = AGG_BP_STATS_DATA_BY_USER_GROUP;
                        break;
                    case 26:
                        sampleCase = AGG_BT_STATS_DATA_BY_USER_GROUP;
                        break;
                    case 27:
                        sampleCase = AGG_BP_STATS_DATA_BY_BP_USER_GROUP;
                        break;
                    case 28:
                        sampleCase = AGG_BT_STATS_DATA_BY_BT_USER_GROUP;
                        break;
                    case 29:
                        sampleCase = AGG_BT_STATS_DATA_BY_BP_BT_USER_GROUP;
                        break;
                    case 30: 
                        sampleCase = AGG_BP_STATS_DATA_BY_APP;
                        break;
                    case 31:
                        sampleCase = AGG_BT_STATS_DATA_BY_APP;
                        break;
                    case 32:
                        sampleCase = AGG_BP_STATS_DATA_BY_APP_USER_GROUP;
                        break;
                    case 33:
                        sampleCase = AGG_BT_STATS_DATA_BY_APP_USER_GROUP;
                        break;
                    case 34:
                        sampleCase = AGG_BP_STATS_DATA_BY_USER;
                        break;
                    case 35:
                        sampleCase = AGG_BT_STATS_DATA_BY_USER;
                        break;
                    default:
                        Util.usage();
                        System.exit(-1);
                        break;                                            
                    }
                }else if (args[i].compareToIgnoreCase("-user") == 0){
                    i++;
                    if (i >= args.length) {
                        Util.usage();
                        System.exit(-1);
                    }
                    user = new String(args[i]);
                    System.out.println("user : " + user);
                }else if (args[i].compareToIgnoreCase("-usergroup") == 0){
                    i++;
                    if (i >= args.length) {
                        Util.usage();
                        System.exit(-1);
                    }
                    userGroup = new String(args[i]);
                    System.out.println("user group : " + userGroup);
                }else if (args[i].compareToIgnoreCase("-appname") == 0) {
                    i++;
                    if (i >= args.length) {
                        Util.usage();
                        System.exit(-1);
                    }
                    application = new String(args[i]);
                    System.out.println("application : " + application);
                }else if (args[i].compareToIgnoreCase("-bpname") == 0) {
                    i++;
                    if (i >= args.length) {
                        Util.usage();
                        System.exit(-1);
                    }
                    businessProcess = new String(args[i]);
                    System.out.println("business process : " + businessProcess);
                }else if (args[i].compareToIgnoreCase("-btname") == 0) {
                    i++;
                    if (i >= args.length) {
                        Util.usage();
                        System.exit(-1);
                    }
                    businessTransaction = new String(args[i]);
                    System.out.println("business transaction : " + businessTransaction);
                }else if (args[i].compareToIgnoreCase("-output") == 0){
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    outPutFileName = new String(args[i]);
                    System.out.println("Export file : " + outPutFileName);
                }else if (args[i].compareToIgnoreCase("-dtype") == 0){
                    i++;
                    if (i >= args.length) {
                        Util.usage();
                        System.exit(-1);
                    }
                    defectType = new String(args[i]);
                    System.out.println("Defect type : " + defectType);
                }else if (args[i].compareToIgnoreCase("-pthroughput") == 0){
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    throughputPercentiles = parsePercentileArg(args[i]);
                    System.out.println("transaction throughput percentile : " + percentilesToString(throughputPercentiles));
                }else if (args[i].compareToIgnoreCase("-ptime") == 0) {
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    timePercentiles = parsePercentileArg(args[i]);
                    System.out.println("transaction time percentile : " + percentilesToString(timePercentiles));
                }else if (args[i].compareToIgnoreCase("-psize") == 0){
                    i++;
                    if (i >= args.length){
                        Util.usage();
                        System.exit(-1);
                    }
                    
                    sizePercentiles = parsePercentileArg(args[i]);
                    System.out.println("transaction size percentile : " + percentilesToString(sizePercentiles));
                }
            }
            
            if (strCEMHost == null){
                System.out.println("Input Error: Missing CEM hostname");
                Util.usage();
                System.exit(-1);
            }else if (username == null){
                System.out.println("Input Error: Missing CEM webservice login username");
                Util.usage();
                System.exit(-1);
            }else if (password == null){
                System.out.println("Input Error: Missing CEM webservice password");
                Util.usage();
                System.exit(-1);
            }else if (strPort == null){
                System.out.println("Input Error: Missing CEM webservice listen port");
                Util.usage();
                System.exit(-1);
            }else if (AggregationType == 0 && sampleNum < 10){
                System.out.println("Input Error: Missing aggregation type");
                Util.usage();
                System.exit(-1);
            }else if (strStartTime == null){
                System.out.println("Input Error: Missing query start time");
                Util.usage();
                System.exit(-1);
            }else if (strEndTime == null){
                System.out.println("Input Error: Missing query end time");
                Util.usage();
                System.exit(-1);
            }else if (sampleCase == null){
                System.out.println("Input Error: Missing command case");
                Util.usage();
                System.exit(-1);
            }else if (outPutFileName == null){
                System.out.println("Input Error: Missing export file name");
                Util.usage();
                System.exit(-1);
            }

            if ((sampleNum == 3 || 
                 sampleNum == 7 || 
                 sampleNum == 9 || 
                 sampleNum == 10 || 
                 sampleNum == 14 ||
                 sampleNum == 23 ||
                 sampleNum == 27) && (businessProcess == null)) {
                System.out.println("Input Error: Missing business process name.");
                Util.usage();
                System.exit(-1);
            }
                
            if ((sampleNum == 4 || 
                 sampleNum == 8 || 
                 sampleNum == 9 || 
                 sampleNum == 11 || 
                 sampleNum == 14 ||
                 sampleNum == 24 ||
                 sampleNum == 28 ||
                 sampleNum == 29) && (businessTransaction == null)) {
                System.out.println("Input Error: Missing business transaction name.");
                Util.usage();
                System.exit(-1);
            }
            
            if ((sampleNum == 5 || 
                 sampleNum == 6 || 
                 sampleNum == 7 || 
                 sampleNum == 8 || 
                 sampleNum == 9 || 
                 sampleNum == 12 || 
                 sampleNum == 14 ||
                 sampleNum == 18 ||
                 sampleNum == 19 ||
                 sampleNum == 25 ||
                 sampleNum == 26 ||
                 sampleNum == 27 ||
                 sampleNum == 28 ||
                 sampleNum == 29 ||
                 sampleNum == 32 ||
                 sampleNum == 33) && (userGroup == null)){
                System.out.println("Input Error: Missing user group name.");
                Util.usage();
                System.exit(-1);
            }
            
            if ((sampleNum == 34 || 
                 sampleNum == 35) && (user == null)){
                System.out.println("Input Error: Missing user name.");
                Util.usage();
                System.exit(-1);
            }
            
            if ((sampleNum == 14 || 
                 sampleNum == 13) && defectType == null){
                System.out.println("Input Error: Missing defect type.");
                Util.usage();
                System.exit(-1);
            }
            
            if ((sampleNum == 16 || 
                 sampleNum == 17 ||
                 sampleNum == 18 ||
                 sampleNum == 19 ||
                 sampleNum == 30 ||
                 sampleNum == 31 ||
                 sampleNum == 32 ||
                 sampleNum == 33) && application == null) {
                System.out.println("Input Error: Missing application.");
                Util.usage();
                System.exit(-1);
            }
            
            // support security webservice
            HttpClient client = new HttpClient();
            String loginPage = "http://" + strCEMHost + ":" + strPort + "/wily/cem/tess/app/j_acegi_security_check";
            
            PostMethod post = new PostMethod(loginPage);
            
            // CEM webservice Login with security
            NameValuePair userdata = new NameValuePair("j_username", username);
            NameValuePair pwddata =  new NameValuePair("j_password", password);
               
            post.addParameter(userdata);
            post.addParameter(pwddata);    
            
            // execute method and handle any erro responses.
            client.executeMethod(post);
            
            String redirectLocation = post.getResponseHeader("location").getValue();
            String LOGIN_ERROR = "login_error";     
            if (redirectLocation.indexOf(LOGIN_ERROR) > -1)
            {
                    System.err.println("Invalid credentials. Please check the username and password.");
                    System.exit(-1);
            }
            
            String jsessionid = redirectLocation.substring(redirectLocation.indexOf('=') + 1,
                     redirectLocation.length());
            
            // create new soWS, coWS, eoWS web service stub instance
            IStatisticsDataOutServiceServiceLocator soWS = new IStatisticsDataOutServiceServiceLocator();
            soWS.setMaintainSession(true);
            String soEndpointURL = "http://" + strCEMHost + ":" + strPort + "/wily/cem/webservices/StatisticsDataOutService";
            java.net.URL soURL = new java.net.URL(soEndpointURL);        
            StatisticsDataOutServiceSoapBindingStub sdo = new StatisticsDataOutServiceSoapBindingStub(soURL, soWS);
            sdo._setProperty(javax.xml.rpc.Stub.SESSION_MAINTAIN_PROPERTY, Boolean.TRUE);
            sdo._setProperty(HTTPConstants.HEADER_COOKIE,"JSESSIONID=" + jsessionid);
            
            IConfigurationDataOutServiceServiceLocator coWS = new IConfigurationDataOutServiceServiceLocator();
            coWS.setMaintainSession(true);
            String coEndpointURL = "http://" + strCEMHost + ":" + strPort + "/wily/cem/webservices/ConfigurationDataOutService";
            java.net.URL coURL = new java.net.URL(coEndpointURL);
            ConfigurationDataOutServiceSoapBindingStub cdo = new ConfigurationDataOutServiceSoapBindingStub(coURL, coWS);
            cdo._setProperty(javax.xml.rpc.Stub.SESSION_MAINTAIN_PROPERTY, Boolean.TRUE);
            cdo._setProperty(HTTPConstants.HEADER_COOKIE,"JSESSIONID=" + jsessionid);
            
            IEventsDataOutServiceServiceLocator eoWS = new IEventsDataOutServiceServiceLocator();
            eoWS.setMaintainSession(true);
            String eoEndpointURL = "http://" + strCEMHost + ":" + strPort + "/wily/cem/webservices/EventsDataOutService";
            java.net.URL eoURL = new java.net.URL(eoEndpointURL);
            EventsDataOutServiceSoapBindingStub edo = new EventsDataOutServiceSoapBindingStub(eoURL, eoWS);
            edo._setProperty(javax.xml.rpc.Stub.SESSION_MAINTAIN_PROPERTY, Boolean.TRUE);
            edo._setProperty(HTTPConstants.HEADER_COOKIE, "JSESSIONID=" + jsessionid);    
            
            DTimeDuration dTimeDuration = new DTimeDuration();
            dTimeDuration.setAggregationInterval(AggregationType);

            Calendar calStart = Calendar.getInstance();
            calStart.setTime(new Date(strStartTime));

            Calendar calEnd = Calendar.getInstance();
            calEnd.setTime(new Date(strEndTime));
            
            dTimeDuration.setStartTime(calStart);
            dTimeDuration.setEndTime(calEnd);
            
            // open the output file
            Util.openDataFile(outPutFileName);
            Util.cemHeaderExportToDataFile(sampleNum, throughputPercentiles, sizePercentiles, timePercentiles);
            
            if (sampleCase.compareToIgnoreCase(ALL_BP_STATS_DATA) == 0){
                Samples.getAllBPStatisticData(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles);            
            }else if (sampleCase.compareToIgnoreCase(ALL_BT_STATS_DATA) == 0){
                Samples.getAllBTStatisticData( 
                        cdo, 
                        sdo, 
                        dTimeDuration,
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles);
            }else if (sampleCase.compareToIgnoreCase(BP_STATS_DATA_BY_BP) == 0){ 
                Samples.getBPStatisticDataByBusinessProcess(
                        cdo, 
                        sdo, 
                        dTimeDuration,
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessProcess);
            }else if (sampleCase.compareToIgnoreCase(BT_STATS_DATA_BY_BT) == 0){ 
                Samples.getBTStatisticDataByBusinessTransaction(
                        cdo,
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessTransaction);
            }else if (sampleCase.compareToIgnoreCase(BP_STATS_DATA_BY_USER_GROUP) == 0) { 
                Samples.getBPStatisticDataByUserGroup( 
                        cdo, 
                        sdo, 
                        dTimeDuration,
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        userGroup);
            }else if (sampleCase.compareToIgnoreCase(BT_STATS_DATA_BY_USER_GROUP) == 0){    
                Samples.getBTStatisticDataByUserGroup(  
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        userGroup);
            }else if(sampleCase.compareToIgnoreCase(BP_STATS_DATA_BY_BP_USER_GROUP) == 0){ 
                Samples.getBPStatisticDataByBusinessProcess_UserGroup(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessProcess, 
                        userGroup);
            }else if (sampleCase.compareToIgnoreCase(BT_STATS_DATA_BY_BT_USER_GROUP) == 0){
                Samples.getBTStatisticDataByBusinessTransaction_UserGroup( 
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessTransaction, 
                        userGroup);
            }else if (sampleCase.compareToIgnoreCase(BT_STATS_DATA_BY_BP_BT_USER_GROUP) == 0){
                Samples.getBTStatisticDataByBusinessProcess_BusinessTransaction_UserGroup(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessProcess, 
                        businessTransaction, 
                        userGroup);
            }else if (sampleCase.compareToIgnoreCase(DEFECTS_DATA_BY_BP) == 0) {
                Samples.getDefectsDataByBusinessProcess(
                        cdo,
                        edo,
                        dTimeDuration,
                        businessProcess);
            }else if (sampleCase.compareToIgnoreCase(DEFECTS_DATA_BY_BT) == 0){
                Samples.getDefectsDataByBusinessTransaction(
                        cdo,
                        edo,
                        dTimeDuration,
                        businessTransaction);
            }else if (sampleCase.compareToIgnoreCase(DEFECTS_DATA_BY_USER_GROUP) == 0){
                Samples.getDefectsdataByUserGroup(
                        cdo,
                        edo,
                        dTimeDuration,
                        userGroup);
            }else if (sampleCase.compareToIgnoreCase(DEFECTS_DATA_BY_TYPE) == 0) {
                Samples.getDefectsDataByDefectType(
                        edo,
                        dTimeDuration,
                        defectType);
            }else if (sampleCase.compareToIgnoreCase(DEFECTS_DATA_BY_BP_BT_USER_GROUP_TYPE) == 0){
                Samples.getDefectsDataByBusinessProcess_BusinessTransaction_UserGroup_Type(
                        cdo,
                        edo,
                        dTimeDuration,
                        businessProcess,
                        businessTransaction,
                        userGroup,
                        defectType);    
            }else if (sampleCase.compareToIgnoreCase(INCIDENTS_DATA_BY_TIME_RANGE) == 0){
                Samples.getIncidentsDataByTimeRange(
                        edo,
                        dTimeDuration);
            }else if (sampleCase.compareToIgnoreCase(BP_STATS_DATA_BY_APP) == 0 ) {
                Samples.getBPStatisticDataByApplication(
                        cdo, 
                        sdo, 
                        application, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles);
            }else if (sampleCase.compareToIgnoreCase(BT_STATS_DATA_BY_APP) == 0 ) {
                Samples.getBTStatisticDataByApplication(
                        cdo, 
                        sdo, 
                        application, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles);
            }else if (sampleCase.compareToIgnoreCase(BP_STATS_DATA_BY_APP_USER_GROUP) == 0) {
                Samples.getBPStatisticDataByApplication_UserGroup(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles, 
                        application, 
                        userGroup);
                
            }else if (sampleCase.compareToIgnoreCase(BT_STATS_DATA_BY_APP_USER_GROUP) == 0) {
                Samples.getBTStatisticDataByApplication_UserGroup(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles, 
                        application, 
                        userGroup);
            } else if (sampleCase.compareToIgnoreCase(AGG_BP_STATS_DATA) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBPStatisticData(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles);       
            } else if (sampleCase.compareToIgnoreCase(AGG_BT_STATS_DATA) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBTStatisticData(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles);       
            } else if (sampleCase.compareToIgnoreCase(AGG_BP_STATS_DATA_BY_BP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBPStatisticDataByBusinessProcess(
                        cdo, 
                        sdo, 
                        dTimeDuration,
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessProcess);
            } else if(sampleCase.compareToIgnoreCase(AGG_BT_STATS_DATA_BY_BT) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBTStatisticDataByBusinessTransaction(
                        cdo,
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessTransaction);
            } else if(sampleCase.compareToIgnoreCase(AGG_BP_STATS_DATA_BY_USER_GROUP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBPStatisticDataByUserGroup( 
                    cdo, 
                    sdo, 
                    dTimeDuration,
                    throughputPercentiles, 
                    sizePercentiles, 
                    timePercentiles,
                    userGroup);
            } else if(sampleCase.compareToIgnoreCase(AGG_BT_STATS_DATA_BY_USER_GROUP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBTStatisticDataByUserGroup(  
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        userGroup);
                
            } else if(sampleCase.compareToIgnoreCase(AGG_BP_STATS_DATA_BY_BP_USER_GROUP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBPStatisticDataByBusinessProcess_UserGroup(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessProcess, 
                        userGroup);
                
            } else if(sampleCase.compareToIgnoreCase(AGG_BT_STATS_DATA_BY_BT_USER_GROUP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBTStatisticDataByBusinessTransaction_UserGroup( 
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessTransaction, 
                        userGroup);
                
            } else if(sampleCase.compareToIgnoreCase(AGG_BT_STATS_DATA_BY_BP_BT_USER_GROUP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBTStatisticDataByBusinessProcess_BusinessTransaction_UserGroup(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        businessProcess, 
                        businessTransaction, 
                        userGroup);
                
            } else if(sampleCase.compareToIgnoreCase(AGG_BP_STATS_DATA_BY_APP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBPStatisticDataByApplication(
                        cdo, 
                        sdo, 
                        application, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles);
                
            } else if(sampleCase.compareToIgnoreCase(AGG_BT_STATS_DATA_BY_APP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBTStatisticDataByApplication(
                        cdo, 
                        sdo, 
                        application, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles);
                
            } else if(sampleCase.compareToIgnoreCase(AGG_BP_STATS_DATA_BY_APP_USER_GROUP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBPStatisticDataByApplication_UserGroup(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles, 
                        application, 
                        userGroup);
                
            } else if(sampleCase.compareToIgnoreCase(AGG_BT_STATS_DATA_BY_APP_USER_GROUP) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBTStatisticDataByApplication_UserGroup(
                        cdo, 
                        sdo, 
                        dTimeDuration, 
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles, 
                        application, 
                        userGroup);
                
            } else if(sampleCase.compareToIgnoreCase(AGG_BP_STATS_DATA_BY_USER) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBPStatisticDataByUser( 
                        cdo, 
                        sdo, 
                        dTimeDuration,
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        user);
            }  else if(sampleCase.compareToIgnoreCase(AGG_BT_STATS_DATA_BY_USER) == 0) {
                Util.adjustAggregateeTimeDuratoin(dTimeDuration);
                Samples.getAggregatedBTStatisticDataByUser( 
                        cdo, 
                        sdo, 
                        dTimeDuration,
                        throughputPercentiles, 
                        sizePercentiles, 
                        timePercentiles,
                        user);
            } 
            
            // close the output file
            Util.closeDataFile();
            
            System.out.println("CEM data exported successfully.");
            
        }catch(Exception e) {
            System.err.println("Catch exception : " + e.getMessage());
            e.printStackTrace(System.out);
        }
    }
    
    private static float[] parsePercentileArg(String argValue) {
        float[] percents = null;
        if(argValue.equalsIgnoreCase("all")) {
            // First, check if it is "all"
            percents = new float[] {
                    0.05f, 0.10f, 0.15f, 0.20f, 0.25f, 
                    0.30f, 0.35f, 0.40f, 0.45f, 0.50f, 
                    0.55f, 0.60f, 0.65f, 0.70f, 0.75f, 
                    0.80f, 0.85f, 0.90f, 0.95f};
        } else {
            // Split the multiple percentages if needed
            if(argValue.contains(";")) {
                String[] percentTokens = argValue.split(";");
                percents = new float[percentTokens.length];
                for(int ti = 0; ti < percentTokens.length; ti ++) {
                    percents[ti] = Float.parseFloat(percentTokens[ti]);
                }
            } else {
                percents = new float[1];
                percents[0] = Float.parseFloat(argValue);
            }
        }
        
        return percents;
    }
    
    private static String percentilesToString(float[] percentiles) {
        StringBuilder sb = new StringBuilder();
        for(int i = 0; i < percentiles.length; i ++) {
            sb.append(percentiles[i]);
            sb.append(" ");
        }
        
        return sb.toString();
    }
}
