/**
 * Created on March 8, 2007
 * 
 * Tool for exporting CEM Business Process, Business Transaction statistic, Defects data and Incidents data through CEM webservices.
 * 
 * @copyright 1999-2007 CA.  All rights reserved.
 */
package com.export;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.List;

import com.ca.wily.cemwebservices.CEMWebServicesException;
import com.ca.wily.cemwebservices.DBusinessProcessStats;
import com.ca.wily.cemwebservices.DBusinessTransactionStats;
import com.ca.wily.cemwebservices.DDefect;
import com.ca.wily.cemwebservices.DIncident;
import com.ca.wily.cemwebservices.DPercentileData;
import com.ca.wily.cemwebservices.DStatistics;
import com.ca.wily.cemwebservices.DTimeDuration;
import static com.export.Constants.*;

/**
 * Contains some utilities in using CEMExportTool.
 * 
 * @author xinyi01
 */
public class Util 
{
    protected static FileOutputStream fileoutputStream = null;
    protected static PrintWriter dataWriter;
    protected static File dataFile;
    
    /**
     * Usage message shows what parameters need be inputed in each sample.
     * @since 1.0
     */
    public static void usage() 
    {
        System.out.println("\nExport tool for retrieving CEM business transaction data write to an text file.\n\n");
        System.out.println("Usage: java -jar CEMExportTool.jar -host <CEMHost> -username <username> -passwrod <password> -port <portnumber> -interval <AggregationType> -start <startTime> -end <endTime> -command <testCase> -bpname <business process name> -btname <business transaction name> -usergroup <user group> -output <outputfile>");
        System.out.println("    -host: cem hostname");
        System.out.println("    -interval: statistics data aggregation type. 1: Hourly 2: Daily 3: Weekly 4: Monthly");
        System.out.println("    -start: start time for query data");
        System.out.println("    -end: end time for query data");
        System.out.println("    -command: sample case");
        System.out.println("    \t 1: all Business Process statistic data");
        System.out.println("    \t 2: all Business Transaction statistic data");
        System.out.println("    \t 3: Business Process statistic data for a specified business process name");
        System.out.println("    \t 4: Business Transaction statistic data for a specified business transaction name");
        System.out.println("    \t 5: Business Process statistic data for a specified user group");
        System.out.println("    \t 6: Business Transaction statistic data for a specified user group");
        System.out.println("    \t 7: Business Process statistic data for a specified user group and specified business process name");
        System.out.println("    \t 8: Business Transaction statistic data for a specified user group and business transaction name");
        System.out.println("    \t 9: Business Transaction statistic data for a specified user group, business process name and business transaction name");
        System.out.println("    \t 10: Defects for specified business process");
        System.out.println("    \t 11: Defects for specified business transaction");
        System.out.println("    \t 12: Defects for specified user group");
        System.out.println("    \t 13: Defects by type");
        System.out.println("    \t 14: Defects for specified user group, business process name, business transaction name and defect type");
        System.out.println("    \t 15: Incidents in certain time range");
        System.out.println("    \t 16: Business Process statistic data for a specified application name");
        System.out.println("    \t 17: Business Transaction statistic data for a specified application name");
        System.out.println("    \t 18: Business Process statistic data for a specified user group and specified application name");
        System.out.println("    \t 19: Business Transaction statistic data for a specified user group and specified application name");
        System.out.println("    \t 21: Aggregated Business Process statistic data for all Business Processes");
        System.out.println("    \t 22: Aggregated Business Transaction statistic data for all Business Transactions");
        System.out.println("    \t 23: Aggregated Business Process statistic data for a specified business process name");
        System.out.println("    \t 24: Aggregated Business Transaction statistic data for a specified business transaction name");
        System.out.println("    \t 25: Aggregated Business Process statistic data for a specified user group");
        System.out.println("    \t 26: Aggregated Business Transaction statistic data for a specified user group");
        System.out.println("    \t 27: Aggregated Business Process statistic data for a specified user group and business process name");
        System.out.println("    \t 28: Aggregated Business Transaction statistic data for a specified user group and business transaction name");
        System.out.println("    \t 29: Aggregated Business Transaction statistic data for a specified user group, business process name and business transaction name");
        System.out.println("    \t 30: Aggregated Business Process statistic data for a specified application name");
        System.out.println("    \t 31: Aggregated Business Transaction statistic data for a specified application name");
        System.out.println("    \t 32: Aggregated Business Process statistic data for a specified user group and application name");
        System.out.println("    \t 33: Aggregated Business Transaction statistic data for a specified user group and application name");
        System.out.println("    \t 34: Aggregated Business Process statistic data for a specified user");
        System.out.println("    \t 35: Aggregated Business Transaction statistic data for a specified user");
        System.out.println("    -user: user name");
        System.out.println("    -usergroup: user group name");
        System.out.println("    -appname: application name");
        System.out.println("    -bpname: business process name");
        System.out.println("    -btname: business transaction name");
        System.out.println("    -pthroughput: percentiles for transaction throughput, default 50%");
        System.out.println("    -ptime: percentiles for transaction time, default 50%");
        System.out.println("    -psize: percentiles for transaction size, default 50%");
        System.out.println("    -dtype: defect type. 1: slow time 2: fast time 3: high throughput 4: low throughput 5: large size");
        System.out.println("                         6: small size 8: http status code 9: missing transaction/component");
        System.out.println("                        10: content error 11: missing response 16: partial response");
        System.out.println("    -output: export file name");
        System.out.println("\n");
        System.out.println("    example: java -jar CEMExportTool.jar -host cembox -username username -password password -port 8080 -interval 3 -start \"1/15/2009 00:00:00\" -end \"1/20/2009 23:59:00\" -command 1 -output data.csv");
        System.out.println(" \nor\n");
        System.out.println("    example: java -jar CEMExportTool.jar -host cembox -username username -password password -port 8008 -interval 3 -start \"1/15/2009 00:00:00\" -end \"1/20/2009 23:59:00\" -command 1 -pthroughput 0.25;0.5;0.75 -psize all -ptime 0.5 -output data.csv");
        System.out.println(" \nor\n");
        System.out.println("    example: java -jar CEMExportTool.jar -host cembox -username username -password password -port 8080 -interval 3 -start \"1/15/2009 00:00:00\" -end \"1/20/2009 23:59:00\" -command 5 -bpname \"shopping cart\" -usergroup \"Unspecified Users\" -output data.csv");
        System.out.println(" \nor\n");
        System.out.println("    example: java -jar CEMExportTool.jar -host cembox -username username -password password -port 8008 -start \"7/15/2009 00:00:00\" -end \"8/15/2009 23:59:00\" -command 21 -pthroughput 0.25;0.5;0.75 -psize 0.5 -ptime 0.5 -output data.csv");
        System.out.println(" \nor\n");
        System.out.println("    example: java -jar CEMExportTool.jar -host cembox -username username -password password -port 8008 -start \"7/15/2009 08:00:00\" -end \"7/15/2009 17:59:00\" -command 21 -output data.csv");
    }
    
    /**
     * Opens a file to write statistic data
     * @param fileName
     *             Name of output file
     * @throws Exception
     * @since 1.0
     */
    public static void openDataFile(String fileName) throws Exception
    {
        if (dataFile == null)
            dataFile = new File(fileName);
        
        if (dataWriter == null)
            dataWriter = new PrintWriter(new OutputStreamWriter(new FileOutputStream(dataFile)), true);
    }
    
    
    /**
     * File close
     * @throws FileNotFoundException
     * @throws IOException
     */
    public static void closeDataFile() throws FileNotFoundException, IOException
    {
        dataWriter.close();
    }
    
    
    /**
     * Writes data into output file with ',' as delimiter
     * @param value
     *             A string value to be written into file
     * @throws Exception
     * @since 1.0
     */
    public static void cemValueExportToDataFile(String value) throws Exception
    {
        if (value != null)
        {
            // If the value contains commas (example 403,405) then
            // we enclose it in double-quotes ("). Otherwise these
            // commas split the data into multiple columns causing
            // the columns to get out of alignment.
            if (value.contains(","))
            {
                String tmpValue = "\"" + value + "\"";
                dataWriter.print(tmpValue + comma);
            }
            else
                dataWriter.print(value + comma);
        }
        else
            dataWriter.print(" " + comma);
        
        if (dataWriter.checkError())
        {
            throw new Exception("Error to write data " + value + " to file");
        }
    }
    
    
    /**
     * Writes the last data of the line into output file
     * @param value
     *             A string value to be written into file
     * @throws Exception
     * @since 1.0
     */
    public static void cemLastValueExportToDataFile(String value) throws Exception
    {
        if (value != null)
        {
            // If the value contains commas (example 403,405) then
            // we enclose it in double-quotes ("). Otherwise these
            // commas split the data into multiple columns causing
            // the columns to get out of alignment.
            if (value.contains(","))
            {
                String tmpValue = "\"" + value + "\"";
                dataWriter.print(tmpValue);
            }
            else
                dataWriter.print(value);
        }
        else
            dataWriter.print(" ");
        
        if (dataWriter.checkError())
        {
            throw new Exception("Error to write data " + value + " to file");
        }
    }
    
    
    /**
     * Writes header into output file
     * @param samplecase
     *             Case number to indicate what the output file header need to be written
     * @throws Exception
     * @since 1.0
     */
    public static void cemHeaderExportToDataFile(int samplecase, float[] throughputPercents, float[] sizePercents, float[] timePercents) throws Exception
    {
        StringBuilder throughputPercentileSb = new StringBuilder();
        if(throughputPercents != null && throughputPercents.length > 0) {
            for(float percent : throughputPercents) {
                throughputPercentileSb.append(TRANSACTION_THROUGHPUT_PERCENTILE);
                throughputPercentileSb.append((int)(percent*100));
                throughputPercentileSb.append("%,");
            }
        } else {
            throughputPercentileSb.append(TRANSACTION_THROUGHPUT_PERCENTILE);
            throughputPercentileSb.append("50%,");
        }
        
        StringBuilder sizePercentileSb = new StringBuilder();
        if(sizePercents != null && sizePercents.length > 0) {
            for(float percent : sizePercents) {
                sizePercentileSb.append(TRANSACTION_SIZE_PERCENTILE);
                sizePercentileSb.append((int)(percent*100));
                sizePercentileSb.append("%,");
            }
        } else {
            sizePercentileSb.append(TRANSACTION_SIZE_PERCENTILE);
            sizePercentileSb.append("50%,");
        }
        
        StringBuilder timePercentileSb = new StringBuilder();
        if(timePercents != null && timePercents.length > 0) {
            for(float percent : timePercents) {
                timePercentileSb.append(TRANSACTION_TIME_PERCENTILE);
                timePercentileSb.append((int)(percent*100));
                timePercentileSb.append("%,");
            }
        } else {
            timePercentileSb.append(TRANSACTION_TIME_PERCENTILE);
            timePercentileSb.append("50%,");
        }

        StringBuilder headerSb = new StringBuilder();
        if (samplecase == 1 || 
            samplecase == 3 || 
            samplecase == 16 || 
            samplecase == 21 || 
            samplecase == 23 || 
            samplecase == 30) {
            // Business Process Statistics Data
            headerSb.append(START_TIME + ",");
            if(samplecase == 21 || 
                    samplecase == 23 || 
                    samplecase == 30) {
                headerSb.append(END_TIME + ",");
            }
            headerSb.append(APPLICATION_ID + ",");
            headerSb.append(APPLICATION_NAME + ",");
            headerSb.append(BUSINESS_PROCESS_ID + ",");
            headerSb.append(BUSINESS_PROCESS_NAME + ",");
            headerSb.append(TOTAL_TRANSACTION_COUNT + ",");
            headerSb.append(OPPORTUNITY_COUNT + ",");
            headerSb.append(BAD_TRANSACTION_COUNT + ",");
            headerSb.append(DEFECT_COUNT + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MAX + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MIN + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_AVG + ",");
            headerSb.append(throughputPercentileSb);
            headerSb.append(TRANSACTION_THROUGHPUT_STD_DEV + ",");
            headerSb.append(TRANSACTION_SIZE_MAX + ",");
            headerSb.append(TRANSACTION_SIZE_MIN + ",");
            headerSb.append(TRANSACTION_SIZE_AVG + ",");
            headerSb.append(sizePercentileSb);
            headerSb.append(TRANSACTION_SIZE_STD_DEV + ",");
            headerSb.append(TRANSACTION_TIME_MAX + ",");
            headerSb.append(TRANSACTION_TIME_MIN + ",");
            headerSb.append(TRANSACTION_TIME_AVG + ",");
            headerSb.append(timePercentileSb);
            headerSb.append(TRANSACTION_TIME_STD_DEV);
        }else if (samplecase == 2 || 
                  samplecase == 4 || 
                  samplecase == 17 || 
                  samplecase == 22 || 
                  samplecase == 24 || 
                  samplecase == 31) {
            // Business Transaction Statistics Data
            headerSb.append(START_TIME + ",");
            if(samplecase == 22 || samplecase == 24 || samplecase == 31) {
                headerSb.append(END_TIME + ",");
            }
            headerSb.append(APPLICATION_ID + ",");
            headerSb.append(APPLICATION_NAME + ",");
            headerSb.append(BUSINESS_PROCESS_ID + ",");
            headerSb.append(BUSINESS_PROCESS_NAME + "," );
            headerSb.append(BUSINESS_TRANSACTION_ID + ","); 
            headerSb.append(BUSINESS_TRANSACTION_NAME + ",");
            headerSb.append(TOTAL_TRANSACTION_COUNT + ",");
            headerSb.append(OPPORTUNITY_COUNT + ",");
            headerSb.append(BAD_TRANSACTION_COUNT + ","); 
            headerSb.append(DEFECT_COUNT + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MAX + ","); 
            headerSb.append(TRANSACTION_THROUGHPUT_MIN + ","); 
            headerSb.append(TRANSACTION_THROUGHPUT_AVG + ","); 
            headerSb.append(throughputPercentileSb); 
            headerSb.append(TRANSACTION_THROUGHPUT_STD_DEV + ","); 
            headerSb.append(TRANSACTION_SIZE_MAX + ","); 
            headerSb.append(TRANSACTION_SIZE_MIN + ","); 
            headerSb.append(TRANSACTION_SIZE_AVG + ","); 
            headerSb.append(sizePercentileSb);
            headerSb.append(TRANSACTION_SIZE_STD_DEV + ",");
            headerSb.append(TRANSACTION_TIME_MAX + ",");
            headerSb.append(TRANSACTION_TIME_MIN + ",");
            headerSb.append(TRANSACTION_TIME_AVG + ",");
            headerSb.append(timePercentileSb);
            headerSb.append(TRANSACTION_TIME_STD_DEV);
        }else if (samplecase == 5 || 
                  samplecase == 7 || 
                  samplecase == 18 || 
                  samplecase == 25 || 
                  samplecase == 27 || 
                  samplecase == 32) {
            // Business Process Statistics Data for User Group
            headerSb.append(START_TIME + ",");
            if(samplecase == 25 || 
                    samplecase == 27 || 
                    samplecase == 32) {
                headerSb.append(END_TIME + ",");
            }
            headerSb.append(APPLICATION_ID + ",");
            headerSb.append(APPLICATION_NAME + ",");
            headerSb.append(BUSINESS_PROCESS_ID + ",");
            headerSb.append(BUSINESS_PROCESS_NAME + ",");
            headerSb.append(USER_GROUP_ID + ",");
            headerSb.append(USER_GROUP_NAME + ",");
            headerSb.append(TOTAL_TRANSACTION_COUNT + ",");
            headerSb.append(OPPORTUNITY_COUNT + ",");
            headerSb.append(BAD_TRANSACTION_COUNT + ",");
            headerSb.append(DEFECT_COUNT + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MAX + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MIN + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_AVG + ",");
            headerSb.append(throughputPercentileSb);
            headerSb.append(TRANSACTION_THROUGHPUT_STD_DEV + ",");
            headerSb.append(TRANSACTION_SIZE_MAX + ",");
            headerSb.append(TRANSACTION_SIZE_MIN + ",");
            headerSb.append(TRANSACTION_SIZE_AVG + ",");
            headerSb.append(sizePercentileSb);
            headerSb.append(TRANSACTION_SIZE_STD_DEV + ",");
            headerSb.append(TRANSACTION_TIME_MAX + ",");
            headerSb.append(TRANSACTION_TIME_MIN + ",");
            headerSb.append(TRANSACTION_TIME_AVG + ",");
            headerSb.append(timePercentileSb);
            headerSb.append(TRANSACTION_TIME_STD_DEV);
        }else if (samplecase == 6 || 
                  samplecase == 8 || 
                  samplecase == 9 || 
                  samplecase == 19 || 
                  samplecase == 26 || 
                  samplecase == 28 || 
                  samplecase == 29 || 
                  samplecase == 33) {
            // Business Transaction Statistics Data for User Group
            headerSb.append(START_TIME + ",");
            if(samplecase == 26 || 
                    samplecase == 28 || 
                    samplecase == 29 || 
                    samplecase ==33) {
                headerSb.append(END_TIME + ",");
            }
            headerSb.append(APPLICATION_ID + ",");
            headerSb.append(APPLICATION_NAME + ",");
            headerSb.append(BUSINESS_PROCESS_ID + ",");
            headerSb.append(BUSINESS_PROCESS_NAME + ",");
            headerSb.append(USER_GROUP_ID + ",");
            headerSb.append(USER_GROUP_NAME + ",");
            headerSb.append(BUSINESS_TRANSACTION_ID + ",");
            headerSb.append(BUSINESS_TRANSACTION_NAME + ",");
            headerSb.append(TOTAL_TRANSACTION_COUNT + ",");
            headerSb.append(OPPORTUNITY_COUNT + ",");
            headerSb.append(BAD_TRANSACTION_COUNT + ",");
            headerSb.append(DEFECT_COUNT + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MAX + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MIN + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_AVG + ",");
            headerSb.append(throughputPercentileSb);
            headerSb.append(TRANSACTION_THROUGHPUT_STD_DEV + ",");
            headerSb.append(TRANSACTION_SIZE_MAX + ",");
            headerSb.append(TRANSACTION_SIZE_MIN + ",");
            headerSb.append(TRANSACTION_SIZE_AVG + ",");
            headerSb.append(sizePercentileSb);
            headerSb.append(TRANSACTION_SIZE_STD_DEV + ",");
            headerSb.append(TRANSACTION_TIME_MAX + ",");
            headerSb.append(TRANSACTION_TIME_MIN + ",");
            headerSb.append(TRANSACTION_TIME_AVG + ",");
            headerSb.append(timePercentileSb);
            headerSb.append(TRANSACTION_TIME_STD_DEV);
        }else if (samplecase == 34 || 
                  samplecase == 36) {
            // Business Process Statistics Data for User
            headerSb.append(START_TIME + ",");
            headerSb.append(END_TIME + ",");
            headerSb.append(APPLICATION_ID + ",");
            headerSb.append(APPLICATION_NAME + ",");
            headerSb.append(BUSINESS_PROCESS_ID + ",");
            headerSb.append(BUSINESS_PROCESS_NAME + ",");
            headerSb.append(USER_ID + ",");
            headerSb.append(USER_NAME + ",");
            headerSb.append(TOTAL_TRANSACTION_COUNT + ",");
            headerSb.append(OPPORTUNITY_COUNT + ",");
            headerSb.append(BAD_TRANSACTION_COUNT + ",");
            headerSb.append(DEFECT_COUNT + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MAX + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MIN + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_AVG + ",");
            headerSb.append(throughputPercentileSb);
            headerSb.append(TRANSACTION_THROUGHPUT_STD_DEV + ",");
            headerSb.append(TRANSACTION_SIZE_MAX + ",");
            headerSb.append(TRANSACTION_SIZE_MIN + ",");
            headerSb.append(TRANSACTION_SIZE_AVG + ",");
            headerSb.append(sizePercentileSb);
            headerSb.append(TRANSACTION_SIZE_STD_DEV + ",");
            headerSb.append(TRANSACTION_TIME_MAX + ",");
            headerSb.append(TRANSACTION_TIME_MIN + ",");
            headerSb.append(TRANSACTION_TIME_AVG + ",");
            headerSb.append(timePercentileSb);
            headerSb.append(TRANSACTION_TIME_STD_DEV);
        } else if (samplecase == 35 || 
                   samplecase == 37) {
            // Business Transaction Statistics Data for User Group
            headerSb.append(START_TIME + ",");
            headerSb.append(END_TIME + ",");
            headerSb.append(APPLICATION_ID + ",");
            headerSb.append(APPLICATION_NAME + ",");
            headerSb.append(BUSINESS_PROCESS_ID + ",");
            headerSb.append(BUSINESS_PROCESS_NAME + ",");
            headerSb.append(USER_ID + ",");
            headerSb.append(USER_NAME + ",");
            headerSb.append(BUSINESS_TRANSACTION_ID + ",");
            headerSb.append(BUSINESS_TRANSACTION_NAME + ",");
            headerSb.append(TOTAL_TRANSACTION_COUNT + ",");
            headerSb.append(OPPORTUNITY_COUNT + ",");
            headerSb.append(BAD_TRANSACTION_COUNT + ",");
            headerSb.append(DEFECT_COUNT + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MAX + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_MIN + ",");
            headerSb.append(TRANSACTION_THROUGHPUT_AVG + ",");
            headerSb.append(throughputPercentileSb);
            headerSb.append(TRANSACTION_THROUGHPUT_STD_DEV + ",");
            headerSb.append(TRANSACTION_SIZE_MAX + ",");
            headerSb.append(TRANSACTION_SIZE_MIN + ",");
            headerSb.append(TRANSACTION_SIZE_AVG + ",");
            headerSb.append(sizePercentileSb);
            headerSb.append(TRANSACTION_SIZE_STD_DEV + ",");
            headerSb.append(TRANSACTION_TIME_MAX + ",");
            headerSb.append(TRANSACTION_TIME_MIN + ",");
            headerSb.append(TRANSACTION_TIME_AVG + ",");
            headerSb.append(timePercentileSb);
            headerSb.append(TRANSACTION_TIME_STD_DEV); 
        } else if (samplecase == 10 || 
                   samplecase == 11 || 
                   samplecase == 12 || 
                   samplecase == 13 || 
                   samplecase == 14) {
            // Defects
            headerSb.append(DEFECT_ID + ",");
            headerSb.append(DEFECT_NUMBER + ",");
            headerSb.append(OCCUR_DATE + ",");
            headerSb.append(DEFECT_VALUE + ",");
            headerSb.append(OBSERVED_VALUE + ",");
            headerSb.append(HTTP_STATUS_CODE + ",");
            headerSb.append(TRANSACTION_SIZE + ",");
            headerSb.append(TRANSACTION_TIME + ",");
            headerSb.append(BIZ_IMPACT + ",");
            headerSb.append(DEFECT_TYPE + ",");
            headerSb.append(USER_IMPORTANCE + ",");
            headerSb.append(DEFECT_IMPORTANCE + ",");
            headerSb.append(TRANSACTION_IMPORTANCE + ",");
            headerSb.append(SERVER_IP + ",");
            headerSb.append(SERFVER_MAC + ",");
            headerSb.append(SERVER_PORT + ",");
            headerSb.append(CLIENT_IP + ",");
            headerSb.append(MISSING_ID + ",");
            headerSb.append(MISSING_NAME + ",");
            headerSb.append(DATA_TYPE + ",");
            headerSb.append(LOGIN_NAME + ",");
            headerSb.append(BUSINESS_PROCESS_NAME + ",");
            headerSb.append(BUSINESS_TRANSACTION_NAME + ",");
            headerSb.append(USER_GROUP_NAME);
        }else if (samplecase == 15){
            // Incidents
            headerSb.append(INCIDENT_ID + ",");
            headerSb.append(INCIDENT_NAME + ",");
            headerSb.append(TICKET_NUMBER + ",");
            headerSb.append(BUSINESS_PROCESS_NAME + ",");
            headerSb.append(BUSINESS_TRANSACTION_NAME + ",");
            headerSb.append(CAUSE + ",");
            headerSb.append(CLOSE_BY + ",");
            headerSb.append(EVIDENCE1 + ",");
            headerSb.append(EVIDENCE2 + ",");
            headerSb.append(EVIDENCE3 + ",");
            headerSb.append(IMPACT_THRESHOLD + ",");
            headerSb.append(LAST_MODIFY + ",");
            headerSb.append(NUMBER_OF_DEFECTS + ",");
            headerSb.append(NUMBER_OF_GROUPS + ",");
            headerSb.append(NUMBER_OF_USERS + ",");
            headerSb.append(RESOLUTION + ",");
            headerSb.append(START_DATE + ",");
            headerSb.append(CLOSE_DATE + ",");
            headerSb.append(STATUS + ",");
            headerSb.append(ASSIGNEE + ",");
            headerSb.append(BIZ_IMPACT + ",");
            headerSb.append(TRIGGER_DATE); 
        }
        
        dataWriter.println(headerSb.toString());
        if (dataWriter.checkError())
            throw new Exception("write header to file error.");
    }
    
    /**
     * Writes Business Transaction statistic data to an output file
     * @param pairs
     *          Array contains business process name, id, transaction name and id
     * @param rs
     *          list of DBusinessTransactionStats objects.
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void writeBusinessTransactionStatisticToDataFile(DTimeDuration dTimeDuration, String[][] pairs, DBusinessTransactionStats result)
        throws CEMWebServicesException,RemoteException,Exception        
    {
            SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy hh:mm:ss aaa");
            if (dTimeDuration != null) {
                cemValueExportToDataFile(formatter.format(dTimeDuration.getStartTime().getTime()));
                cemValueExportToDataFile(formatter.format(dTimeDuration.getEndTime().getTime()));
            } else if (result.getIntervalStartTime() != null) {
                cemValueExportToDataFile(formatter.format(result.getIntervalStartTime().getTime()));
            } else {
                cemValueExportToDataFile(null);
            }
            
            for (int j = 0; j < pairs.length; j++){
                cemValueExportToDataFile(pairs[j][0]);
                cemValueExportToDataFile(pairs[j][1]);
            }
            
            cemValueExportToDataFile(String.valueOf(result.getTotalTransactionCount()));
            cemValueExportToDataFile(String.valueOf(result.getOpportunityCount()));
            cemValueExportToDataFile(String.valueOf(result.getBadTransactionCount()));
            cemValueExportToDataFile(String.valueOf(result.getDefectCount()));
                
            DStatistics statis = result.getStatistics();
            
            // transaction throughput stats info        
            cemValueExportToDataFile(String.valueOf(statis.getTransactionThroughputMax()));
            cemValueExportToDataFile(String.valueOf(statis.getTransactionThroughputMin()));
            cemValueExportToDataFile(String.valueOf(statis.getTransactionThroughputAverage()));
            // Must be ordered by the percents;
            for(DPercentileData percentile : statis.getTransactionThroughputByPercentiles()) {
                cemValueExportToDataFile(String.valueOf(percentile.getValue()));
            }
            cemValueExportToDataFile(String.valueOf(statis.getTransactionThroughputStdDeviation()));
            
            // transaction size stats info
            cemValueExportToDataFile(String.valueOf(statis.getTransactionSizeMax()));
            cemValueExportToDataFile(String.valueOf(statis.getTransactionSizeMin()));
            cemValueExportToDataFile(String.valueOf(statis.getTransactionSizeAverage()));
            // Must be ordered by the percents
            for(DPercentileData percentile : statis.getTransactionSizeByPercentiles()) {
                cemValueExportToDataFile(String.valueOf(percentile.getValue()));
            }
            cemValueExportToDataFile(String.valueOf(statis.getTransactionSizeStdDeviation()));
            
            // transaction time stats info
            cemValueExportToDataFile(String.valueOf(statis.getTransactionTimeMax()));
            cemValueExportToDataFile(String.valueOf(statis.getTransactionTimeMin()));
            cemValueExportToDataFile(String.valueOf(statis.getTransactionTimeAverage()));
            // Must be ordered by the percents
            for(DPercentileData percentile : statis.getTransactionTimeByPercentiles()) {
                cemValueExportToDataFile(String.valueOf(percentile.getValue()));
            }
            cemLastValueExportToDataFile(String.valueOf(statis.getTransactionTimeStdDeviation()));
            
            dataWriter.println();
            if (dataWriter.checkError())
                throw new Exception();
    }
        
    /**
     * Writes Business Transaction statistic data to an output file
     * @param pairs
     *             Array contains business process name, id, transaction name and id
     * @param rs
     *             list of DBusinessTransactionStats objects.
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void writeBusinessTransactionStatisticToDataFile(String[][] pairs, List<DBusinessTransactionStats> rs)
        throws CEMWebServicesException,RemoteException,Exception        
    {
        for (int i = 0; i < rs.size(); i++) {
            DBusinessTransactionStats stats = rs.get(i);
            writeBusinessTransactionStatisticToDataFile(null, pairs, stats);
        }
    }
    
       /**
     * Writes Business Process statistic data to an output file
     * @param pairs
     *          Array contains business process name, id, transaction name and id
     * @param result
     *          a DBusinessProcessStats object.
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void writeBusinessProcessStatisticToDataFile(DTimeDuration dTimeDuration, String[][] pairs, DBusinessProcessStats result)
        throws CEMWebServicesException,RemoteException,Exception
    {
        SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy hh:mm:ss aaa");
        if (dTimeDuration != null) {
            cemValueExportToDataFile(formatter.format(dTimeDuration.getStartTime().getTime()));
            cemValueExportToDataFile(formatter.format(dTimeDuration.getEndTime().getTime()));
        } else if (result.getIntervalStartTime() != null)
            cemValueExportToDataFile(formatter.format(result.getIntervalStartTime().getTime()));
        else
            cemValueExportToDataFile(null);
        
        for (int j = 0; j < pairs.length; j++){
            cemValueExportToDataFile(pairs[j][0]);
            cemValueExportToDataFile(pairs[j][1]);
        }

        cemValueExportToDataFile(String.valueOf(result.getTotalTransactionCount()));
        cemValueExportToDataFile(String.valueOf(result.getOpportunityCount()));
        cemValueExportToDataFile(String.valueOf(result.getBadTransactionCount()));
        cemValueExportToDataFile(String.valueOf(result.getDefectCount()));
            
        DStatistics statis = result.getStatistics();
        
        // transaction throughput stats info        
        cemValueExportToDataFile(String.valueOf(statis.getTransactionThroughputMax()));
        cemValueExportToDataFile(String.valueOf(statis.getTransactionThroughputMin()));
        cemValueExportToDataFile(String.valueOf(statis.getTransactionThroughputAverage()));
        // Must be ordered by the percents
        for(DPercentileData percentile : statis.getTransactionThroughputByPercentiles()) {
            cemValueExportToDataFile(String.valueOf(percentile.getValue()));
        }
        cemValueExportToDataFile(String.valueOf(statis.getTransactionThroughputStdDeviation()));
        
        // transaction size stats info
        cemValueExportToDataFile(String.valueOf(statis.getTransactionSizeMax()));
        cemValueExportToDataFile(String.valueOf(statis.getTransactionSizeMin()));
        cemValueExportToDataFile(String.valueOf(statis.getTransactionSizeAverage()));
        // Must be ordered by the percents
        for(DPercentileData percentile : statis.getTransactionSizeByPercentiles()) {
            cemValueExportToDataFile(String.valueOf(percentile.getValue()));
        }
        cemValueExportToDataFile(String.valueOf(statis.getTransactionSizeStdDeviation()));
        
        // transaction time stats info
        cemValueExportToDataFile(String.valueOf(statis.getTransactionTimeMax()));
        cemValueExportToDataFile(String.valueOf(statis.getTransactionTimeMin()));
        cemValueExportToDataFile(String.valueOf(statis.getTransactionTimeAverage()));
        // Must be ordered by the percents
        for(DPercentileData percentile : statis.getTransactionTimeByPercentiles()) {
            cemValueExportToDataFile(String.valueOf(percentile.getValue()));
        }
        cemLastValueExportToDataFile(String.valueOf(statis.getTransactionTimeStdDeviation()));
        
        dataWriter.println();
        if (dataWriter.checkError())
            throw new Exception();
    }
    
    /**
     * Writes Business Process statistic data to an output file
     * @param pairs
     *             Array contains business process name, id, transaction name and id
     * @param rs
     *             list of DBusinessProcessStats objects.
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void writeBusinessProcessStatisticToDataFile(String[][] pairs, List<DBusinessProcessStats>rs)
        throws CEMWebServicesException,RemoteException,Exception
    {
        for (int i = 0; i < rs.size(); i++) {
            DBusinessProcessStats stats = rs.get(i);
            writeBusinessProcessStatisticToDataFile(null, pairs, stats);
        }
    }
    
    
    /**
     * Writes Defects data into file
     * @param rs
     *             List of DDefect objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void writeDefectsToDataFile(List<DDefect>rs) 
    throws CEMWebServicesException,RemoteException,Exception
    {
        SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy hh:mm:ss aaa");
        for (int i = 0; i < rs.size(); i++){
            DDefect defect = rs.get(i); 
            cemValueExportToDataFile(String.valueOf(defect.getId()));
            cemValueExportToDataFile(String.valueOf(defect.getDefectNumber()));
            if (defect.getOccurDate() != null)
                cemValueExportToDataFile(String.valueOf(formatter.format(defect.getOccurDate().getTime())));
            else
                cemValueExportToDataFile(null);
            
            cemValueExportToDataFile(defect.getDefectValue());
            cemValueExportToDataFile(defect.getObservedValue());
            cemValueExportToDataFile(defect.getHttpStatusCode());
            cemValueExportToDataFile(String.valueOf(defect.getTranSize()));
            cemValueExportToDataFile(String.valueOf(defect.getTranTime()));
            cemValueExportToDataFile(String.valueOf(defect.getBizImpact()));
            cemValueExportToDataFile(String.valueOf(defect.getDefectType()));
            cemValueExportToDataFile(String.valueOf(defect.getUserImportance()));
            cemValueExportToDataFile(String.valueOf(defect.getDefectImportance()));
            cemValueExportToDataFile(String.valueOf(defect.getTranImportance()));
            cemValueExportToDataFile(defect.getServerIpAddress());
            cemValueExportToDataFile(defect.getServerMacAddress());
            cemValueExportToDataFile(String.valueOf(defect.getServerPort()));
            cemValueExportToDataFile(defect.getClientIpAddress());
            cemValueExportToDataFile(String.valueOf(defect.getMissingId()));
            cemValueExportToDataFile(defect.getMissingName());
            cemValueExportToDataFile(String.valueOf(defect.getDataType()));
            cemValueExportToDataFile(defect.getLoginName());
            cemValueExportToDataFile(defect.getBusinessProcessName());
            cemValueExportToDataFile(defect.getBusinessTransactionName());
            cemLastValueExportToDataFile(defect.getUserGroupName());
            
            dataWriter.println();
            if (dataWriter.checkError())
                throw new Exception();
        }
    }
    
    
    /**
     * Writes Defects data filted by business transaction, user group and defect type
     * @param rs
     *             List of DDefect objects
     * @param bt
     *             business transaction
     * @param ug
     *             user group
     * @param type
     *             defect type
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void writeDefectsToDataFileByFilter(List<DDefect>rs, String bt, String ug, String type)
    throws CEMWebServicesException,RemoteException,Exception
    {
        SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy hh:mm:ss aaa");
        for (int i = 0; i < rs.size(); i++){
            DDefect defect = rs.get(i);
                
            if ( bt.compareTo(defect.getBusinessTransactionName()) == 0 &&
                 ug.compareTo(defect.getUserGroupName()) == 0        &&
                 type.compareTo(String.valueOf(defect.getDefectType())) == 0 )
            {
                cemValueExportToDataFile(String.valueOf(defect.getId()));
                cemValueExportToDataFile(String.valueOf(defect.getDefectNumber()));
                if (defect.getOccurDate() != null)
                    cemValueExportToDataFile(String.valueOf(formatter.format(defect.getOccurDate().getTime())));
                else
                    cemValueExportToDataFile(null);
                    
                cemValueExportToDataFile(defect.getDefectValue());
                cemValueExportToDataFile(defect.getObservedValue());
                cemValueExportToDataFile(defect.getHttpStatusCode());
                cemValueExportToDataFile(String.valueOf(defect.getTranSize()));
                cemValueExportToDataFile(String.valueOf(defect.getTranTime()));
                cemValueExportToDataFile(String.valueOf(defect.getBizImpact()));
                cemValueExportToDataFile(String.valueOf(defect.getDefectType()));
                cemValueExportToDataFile(String.valueOf(defect.getUserImportance()));
                cemValueExportToDataFile(String.valueOf(defect.getDefectImportance()));
                cemValueExportToDataFile(String.valueOf(defect.getTranImportance()));
                cemValueExportToDataFile(defect.getServerIpAddress());
                cemValueExportToDataFile(defect.getServerMacAddress());
                cemValueExportToDataFile(String.valueOf(defect.getServerPort()));
                cemValueExportToDataFile(defect.getClientIpAddress());
                cemValueExportToDataFile(String.valueOf(defect.getMissingId()));
                cemValueExportToDataFile(defect.getMissingName());
                cemValueExportToDataFile(String.valueOf(defect.getDataType()));
                cemValueExportToDataFile(defect.getLoginName());
                cemValueExportToDataFile(defect.getBusinessProcessName());
                cemValueExportToDataFile(defect.getBusinessTransactionName());
                cemLastValueExportToDataFile(defect.getUserGroupName());
                
                dataWriter.println();
                if (dataWriter.checkError())
                    throw new Exception();
            }
        }
    }
    
    
    /**
     * Writes Incidients into output file
     * @param rs
     *             List of DIncident objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void writeIncidentsToDataFile(List<DIncident> rs) 
    throws CEMWebServicesException,RemoteException,Exception
    {
        SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy hh:mm:ss aaa");
        for (int i = 0; i < rs.size(); i++) {
            DIncident incident = rs.get(i);
            cemValueExportToDataFile(String.valueOf(incident.getId()));
            cemValueExportToDataFile(incident.getName());
            cemValueExportToDataFile(incident.getTicketNumber());
            cemValueExportToDataFile(incident.getBusinessProcessName());
            cemValueExportToDataFile(incident.getBusinessTransactionName());
            cemValueExportToDataFile(incident.getCause());
            cemValueExportToDataFile(incident.getClosedBy());
            cemValueExportToDataFile(incident.getEvidence1());
            cemValueExportToDataFile(incident.getEvidence2());
            cemValueExportToDataFile(incident.getEvidence3());
            cemValueExportToDataFile(String.valueOf(incident.getImpactThreshold()));
            if (incident.getLastModified() != null)
                cemValueExportToDataFile(String.valueOf(formatter.format(incident.getLastModified().getTime())));
            else
                cemValueExportToDataFile(null);
            
            cemValueExportToDataFile(String.valueOf(incident.getNumberOfDefects()));
            cemValueExportToDataFile(String.valueOf(incident.getNumberOfGroups()));
            cemValueExportToDataFile(String.valueOf(incident.getNumberOfUsers()));
            cemValueExportToDataFile(incident.getResolution());
            if (incident.getStartDate() != null)
                cemValueExportToDataFile(String.valueOf(formatter.format(incident.getStartDate().getTime())));
            else
                cemValueExportToDataFile(null);
            
            if (incident.getCloseDate() != null)
                cemValueExportToDataFile(String.valueOf(formatter.format(incident.getCloseDate().getTime())));
            else
                cemValueExportToDataFile(null);
            cemValueExportToDataFile(String.valueOf(incident.getStatus()));
            cemValueExportToDataFile(incident.getAssignee());
            cemValueExportToDataFile(String.valueOf(incident.getBizImpact()));
            
            if (incident.getTriggerDate() != null)
                cemLastValueExportToDataFile(String.valueOf(formatter.format(incident.getTriggerDate().getTime())));
            else
                cemLastValueExportToDataFile(null);
            
            dataWriter.println();
            if (dataWriter.checkError())
                throw new Exception();
        }    
    }
    
    public static void adjustAggregateeTimeDuratoin(DTimeDuration dTimeDuration) {
        Calendar startTime = dTimeDuration.getStartTime();
        Calendar endTime = dTimeDuration.getEndTime();
            if(startTime.get(Calendar.YEAR) == endTime.get(Calendar.YEAR)&&
               startTime.get(Calendar.MONTH) == endTime.get(Calendar.MONTH) &&
               startTime.get(Calendar.DAY_OF_MONTH) == endTime.get(Calendar.DAY_OF_MONTH) ) {
                // aggregate from hourly data
                dTimeDuration.setAggregationInterval(DTimeDuration.HOURLY);
            } else {
                // aggregate from daily data
                dTimeDuration.setAggregationInterval(DTimeDuration.DAILY);
                
                // adjust the start time to 00:00:00
                // and end time to 23:59:59
                startTime.set(Calendar.HOUR_OF_DAY, 0);
                startTime.set(Calendar.MINUTE, 0);
                startTime.set(Calendar.SECOND, 0);
                startTime.set(Calendar.MILLISECOND, 0);
                dTimeDuration.setStartTime(startTime);
                
                endTime.set(Calendar.HOUR_OF_DAY, 23);
                endTime.set(Calendar.MINUTE, 59);
                endTime.set(Calendar.SECOND, 59);
                endTime.set(Calendar.MILLISECOND, 0);
                dTimeDuration.setEndTime(endTime);
            }
        
    }
}

